package com.high.pi_framework.base

import android.app.Activity
import android.app.AlarmManager
import android.app.PendingIntent
import android.content.*
import android.os.Bundle
import android.text.TextUtils
import android.widget.Toast
import com.high.pi_framework.Util.AndroidBug5497Workaround
import com.high.pi_framework.Util.AndroidWorkaround
import com.high.pi_framework.Util.ToastManager
import com.high.pi_framework.framework.JSBridge
import com.high.pi_framework.webview.YNWebView
import kotlin.system.exitProcess

abstract class BaseActivity : Activity(), BaseView {

    var isHome: Boolean = false

    var ynWebView = YNWebView()

    private var registered = false

    /**
     * Get the layout resource from XML.
     *
     * @return layout resource from XML.
     */
    protected abstract val layoutResources: Int

    /**
     * mBackgroundReceiver is a listener to listen the application's lifecycle
     * when user lock the screen and press home key,the event will be push to ts.
     */


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        if (NO_LAYOUT != layoutResources) {
            this.setContentView(layoutResources)
            this.setFitSystemWindows()
            this.initViews()
        }
        if (!registered) {
            registerAppLifeListener()
            registered = true
        }
        this.initData()
    }


    override fun onResume() {
        super.onResume()
        val intentFilter = IntentFilter()
        intentFilter.addAction("restart_action")
        registerReceiver(loginReceiver, intentFilter)
    }

    override fun onStop() {
        super.onStop()
        unregisterReceiver(loginReceiver)
    }

    override fun onDestroy() {
        if (registered) {
            try {
                unregisterReceiver(mBackgroundReceiver)
            } catch (e: IllegalArgumentException) {
                e.printStackTrace()
            }
            registered = false
        }
        super.onDestroy()
    }

    private fun setFitSystemWindows() {
        if (AndroidWorkaround.checkDeviceHasNavigationBar(this)) {
            AndroidWorkaround.assistActivity(findViewById(android.R.id.content))
        } else {
            AndroidBug5497Workaround.assistActivity(this)
        }
    }

    /**
     * As the method name said,this method is used to initialize views on this activity.
     */
    protected abstract fun initViews()

    /**
     * Initialize basic data.
     */
    protected abstract fun initData()

    /**
     * Show the toast tip
     *
     * @param resId the message's id of what would you like to tip
     */
    override fun onToast(resId: Int) {
        this.onToast(resources.getString(resId))
    }

    /**
     * Show the toast tip
     *
     * @param msg the message of what would you like to tip
     */
    override fun onToast(msg: String) {
        this.onToast(msg, Toast.LENGTH_SHORT)
    }

    /**
     * Show the toast tip
     *
     * @param msg      the message of what would you like to tip
     * @param duration the message last time.
     */
    override fun onToast(msg: String, @ToastManager.Duration duration: Int) {
        ToastManager.toast(this, msg, duration)
    }

    /**
     * Show the toast tip
     *
     * @param msg      the message's id of what would you like to tip.
     * @param duration the message last time.
     */
    override fun onToast(msg: Int, duration: Int) {
        this.onToast(resources.getString(msg), duration)
    }

    /**
     * Hide the content being prompted.
     */
    override fun onHideToast() {
        ToastManager.hideToast()
    }

    /**
     * Register the broadcast receiver.
     */
    private fun registerAppLifeListener() {
        val filter = IntentFilter()
        filter.addAction(Intent.ACTION_SCREEN_ON)//亮屏
        filter.addAction(Intent.ACTION_SCREEN_OFF)//锁屏
        filter.addAction(Intent.ACTION_USER_PRESENT)//解锁屏幕
        filter.addAction(Intent.ACTION_CLOSE_SYSTEM_DIALOGS)
//        filter.addAction(ConnectivityManager.CONNECTIVITY_ACTION) // 监测网络  //在android N版本上指出，该方法现在只有app在前台时可以接收到
        registerReceiver(mBackgroundReceiver, filter)
    }


    /**
     * App已经进入后台
     */
    private fun gotoBackground() {
        JSBridge.sendJS(ynWebView,"PI_App",ON_BACK_PRESSED, arrayOf("App进入后台"))
    }

    /**
     * App进入前台
     */
    private fun gotoForeground() {
        isHome = false
        JSBridge.sendJS(ynWebView,"PI_App",ON_APP_RESUMED, arrayOf("App进入前台"))
    }

    private var loginReceiver = object : BroadcastReceiver(){
        override fun onReceive(context: Context?, intent: Intent?) {
            val action = intent?.action ?: return
            when (action) {
                "restart_action" -> {
                    val mStartActivity : Intent = context!!.packageManager.getLaunchIntentForPackage(context.packageName)!!
                    val mPendingIntentId : Int  = 123456
                    val mPendingIntent : PendingIntent = PendingIntent.getActivity(context, mPendingIntentId, mStartActivity, PendingIntent.FLAG_CANCEL_CURRENT)
                    val mgr : AlarmManager = context.getSystemService(Context.ALARM_SERVICE) as AlarmManager
                    mgr.set(AlarmManager.RTC, System.currentTimeMillis() + 50, mPendingIntent)
                    exitProcess(0)
                }
            }
        }
    }


    private val mBackgroundReceiver = object : BroadcastReceiver() {
        override fun onReceive(context: Context, intent: Intent) {
            val action = intent.action
            if (TextUtils.isEmpty(action)) return
            when (action) {
                //亮屏
                Intent.ACTION_SCREEN_ON -> { }
                //锁屏
                Intent.ACTION_SCREEN_OFF -> {
                    if (isHome) return
                    gotoBackground()
                }
                //解锁屏幕
                Intent.ACTION_USER_PRESENT -> {
                    if (isHome) return
                    gotoForeground()
                }
                Intent.ACTION_CLOSE_SYSTEM_DIALOGS -> {
                    val reason = intent.getStringExtra(SYSTEM_DIALOG_REASON_KEY)
                    if (null != reason) {
                        if (reason.equals(SYSTEM_DIALOG_REASON_HOME_LONG_KEY,true)) {
                            return
                        }
                        else if (reason.equals(SYSTEM_DIALOG_REASON_HOME_KEY,true)) {
                            if (isHome) return
                            isHome = true
                            gotoBackground()
                        }
                    }
                }
//                //网络状态
//                ConnectivityManager.CONNECTIVITY_ACTION -> {
//                    val connMgr = context.getSystemService(Context.CONNECTIVITY_SERVICE) as ConnectivityManager
//                    val wifiNetworkInfo = connMgr.getNetworkInfo(ConnectivityManager.TYPE_WIFI)
//                    val dataNetworkInfo = connMgr.getNetworkInfo(ConnectivityManager.TYPE_MOBILE)
//                    if (wifiNetworkInfo.isConnected)JSBridge.sendJS(ynWebView, "PI_Activity", ON_Network_Changed, arrayOf(wifiNetworkInfo.subtypeName))
//                    else if (dataNetworkInfo.isConnected)JSBridge.sendJS(ynWebView, "PI_Activity", ON_Network_Changed, arrayOf(wifiNetworkInfo.subtypeName))
//                    else JSBridge.sendJS(ynWebView, "PI_Activity", ON_Network_Changed, arrayOf(""))
//                }
            }
        }
    }

    companion object {
        const val ON_APP_RESUMED = "onResumed"
        const val ON_BACK_PRESSED = "onBackPressed"
        const val ON_Network_Changed = "onNetworkChanged"
        private const val SYSTEM_DIALOG_REASON_KEY = "reason"
        private const val SYSTEM_DIALOG_REASON_HOME_KEY = "homekey"
        private const val SYSTEM_DIALOG_REASON_HOME_LONG_KEY = "recentapps"
        protected const val NO_LAYOUT = -1//No Layout Resources.
    }
}