package com.high.pi_framework.framework

import android.app.Activity
import android.util.Base64
import android.util.Log
import android.webkit.JavascriptInterface
import com.high.pi_framework.Util.FileUtil
import com.high.pi_framework.module.AppUpdater
import com.high.pi_framework.webview.YNWebView
import okhttp3.*
import org.json.JSONArray
import org.json.JSONObject
import java.io.File
import java.io.IOException
import java.util.concurrent.TimeUnit
import kotlin.math.log

class JSIntercept(ynWebView: YNWebView) {
    private val mActivity = ynWebView.getEnv(ynWebView.ACTIVITY) as Activity
    val bootPath: String = mActivity.filesDir.absolutePath
    val htmlPath: String = "$bootPath/html"
    val apkPath: String  = "$bootPath/apk_back"
    var isUpdate: Int = 0
    var name: String = ""
    var yn : YNWebView = ynWebView

    @JavascriptInterface
    fun restartApp(restartPath: String) {
        //H5更新重启
        val url = restartPath
        var content = FileUtil.readFile(this.htmlPath + url)
        if (content == "") {
            val stream = mActivity.getAssets().open(url.substring(1))
            content = FileUtil.readFile(stream)
        }
        if (content != "") {
            mActivity.runOnUiThread { yn.loadDataWithBaseUrl("file:///android_asset" + url, content); }
        } else {
            Log.d("JSIntercept", "loadUrl Error!!!");
        }
    }

    @JavascriptInterface
    fun getAppVersion(listenerID: Int) {
        var version = ""
        var pkg = ""
        try {
            val pm = mActivity.applicationContext.packageManager
            val info = pm.getPackageInfo(mActivity.applicationContext.packageName, 0)
            version = info.versionName
            pkg = mActivity.packageName
        } catch (e: Exception) {
            e.printStackTrace()
        }
        val func = String.format("window.handle_jsintercept_callback(%d, true, '%s', '%s', %d)", listenerID, pkg, version, isUpdate)
        mActivity.runOnUiThread(CallJSRunnable(func, yn.getWeb("")))
    }

    @JavascriptInterface
    fun updateFinish(listenerID: Int){
        isUpdate = 0
        val versionPath = this.apkPath + "/apkversion.txt"
        val f = File(versionPath)
        f.writeText(name)
        val func = String.format("window.handle_jsintercept_callback(%d, true)",listenerID)
        mActivity.runOnUiThread(CallJSRunnable(func, yn.getWeb("")))
    }

    @JavascriptInterface
    fun updateApp(cbID: Int, url: String,listenerID: Int) {
        val updater = AppUpdater(yn, listenerID, cbID)
        updater.updateApp(url)
    }

    @JavascriptInterface
    fun getBootFiles(listenerID: Int, pathsStr: String) {
        val result = JSONArray()
        try {
            val paths = JSONArray(pathsStr)
            for (i in 0 until paths.length()) {
                val path = paths.get(i) as String
                val fullPath = "$htmlPath/$path"
                var str = ""
                val b = FileUtil.readFileToData(fullPath);
                if (b != null) {
                    str = Base64.encodeToString(b, Base64.NO_WRAP)
                }
                result.put(str)
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
        val func = String.format("window.handle_jsintercept_callback(%d, true, '%s')", listenerID, result.toString())
        mActivity.runOnUiThread(CallJSRunnable(func, yn.getWeb("")))
    }

    @JavascriptInterface
    fun saveFile(path: String, base64Str: String, listenerID: Int) {
        var path = path
        try {
            if (path.contains(".depend")) {
                path = path.replace(".depend", "depend")
            }

            val fullPath = "$htmlPath/$path"
            val content = Base64.decode(base64Str, Base64.NO_WRAP)
            FileUtil.writeFile(fullPath, content, false)

            Log.d("Intercept", "JSIntercept.saveFile: $fullPath")
            val func = String.format("window.handle_jsintercept_callback(%d, true)", listenerID)
            mActivity.runOnUiThread(CallJSRunnable(func, yn.getWeb("")))

        } catch (e: Exception) {
            e.printStackTrace()
        }
    }

    @JavascriptInterface
    fun getIndexHtml(timeout: Int){
        if (mActivity is NewWebViewActivity){
            val url:String = mActivity.url!!
            val okHttpClient = OkHttpClient.Builder().connectTimeout(timeout.toLong(), TimeUnit.SECONDS).writeTimeout(timeout.toLong(), TimeUnit.SECONDS).readTimeout(timeout.toLong(),
                TimeUnit.SECONDS).build()
            val request = Request.Builder().url(url).get().build()
            val call = okHttpClient.newCall(request)
            call.enqueue(object : Callback {
                override fun onResponse(call: Call?, response: Response?) {
                    var func:String = "window.handle_event_call(0)"
                    if(response!!.code() == 200){
                        func = String.format("window.handle_event_call(0)")
                        mActivity.injectContent = response.body()!!.string()
                    }
                    else func = String.format("window.handle_event_call(-1)")
                    mActivity.runOnUiThread(CallJSRunnable(func, yn.getWeb("")))
                }
                override fun onFailure(call: Call?, e: IOException?) {
                    val func:String = String.format("window.handle_event_call(-1)")
                    mActivity.runOnUiThread(CallJSRunnable(func, yn.getWeb("")))
                }
            })
        }
    }

    @JavascriptInterface
    fun reloadGame(){
        if (mActivity is NewWebViewActivity){
            var url = mActivity.url.toString()
            val gameJsonStream = mActivity.assets.open("game.json")
            val gameJson = FileUtil.readFile(gameJsonStream)
            val gameObject = JSONObject(gameJson)
            var bool = false
            while (gameObject.keys().hasNext()){
                if (gameObject.keys().next() == mActivity.tag){
                    bool = true
                    break
                }
            }
            if (bool){
                url = gameObject[mActivity.tag].toString()
                mActivity.runOnUiThread { yn.loadDataWithBaseUrl("file:///android_asset" + url, mActivity.injectContent!!); }
            } else {
                mActivity.runOnUiThread { yn.loadDataWithBaseUrl( url, mActivity.injectContent!!); }
            }
        }
    }

    @JavascriptInterface
    fun goBackWallet(){
        if (mActivity is NewWebViewActivity){
            mActivity.closeActivicty()
        }
    }

}