package com.high.pi_framework.framework

import android.content.*
import android.os.Bundle
import android.os.IBinder
import android.util.Log
import android.view.WindowManager
import android.widget.RelativeLayout
import com.high.pi_framework.R
import com.high.pi_framework.Util.FileUtil
import com.high.pi_framework.Util.PrefMgr
import com.high.pi_framework.Util.ViewUtil
import com.high.pi_framework.base.BaseWebView
import com.high.pi_framework.module.LocalLanguageMgr
import com.high.pi_framework.webview.YNWebView
import com.high.pi_service.piservice
import com.high.pi_service.piserviceCallBack
import com.high.pi_service.utils.TextUtil
import kotlinx.android.synthetic.main.layout_fake_status_bar_view.*
import org.json.JSONObject
import java.io.File
import android.content.IntentFilter
import android.content.Intent

class WebViewActivity : BaseWebView() {
    private var chooceAcPath: String? = null
    private var tag: String? = null
    private lateinit var mJsIntercept: JSIntercept
    private lateinit var mRlRootView: RelativeLayout
    private var ps: piservice? = null
    private val conn = webViewConn()
    override val layoutResources: Int get() = R.layout.activity_webview
    var url: String? = null
    var isWallet: Boolean? = false


    //============life==================
    override fun onCreate(savedInstanceState: Bundle?) {
        Log.d("webViewActivity","Life with onCreate..")
        tag = resources.getString(R.string.init_name)
        chooceAcPath = filesDir.absolutePath + "/chooseAc.json"
        bindYnWebView()
        rsgService()
        if(intent?.getStringExtra("webUrl") != null ) {
            url = intent?.getStringExtra("webUrl")
            Log.d("webUrl",url)
        }
        super.onCreate(savedInstanceState)

    }

    override fun initViews() {
        mRlRootView = findViewById(R.id.app_main_rl_root_view)
        mRlRootView.removeAllViews()
        ynWebView.addYnWebView(mRlRootView)
        status_bar.layoutParams.height = ViewUtil.getStatusBarHeight(this).toInt()
    }


    override fun initData() {
        window.addFlags(WindowManager.LayoutParams.FLAG_TRANSLUCENT_STATUS)// 在setContentView之后，适配顶部状态栏
        window.addFlags(WindowManager.LayoutParams.FLAG_TRANSLUCENT_NAVIGATION)// 适配底部导航栏
        mJsIntercept = ynWebView.addJavaScriptInterface( mRlRootView)
        addJEV(this)
        LocalLanguageMgr(ynWebView).setAppLanguage(
            PrefMgr.getInstance(this).appLan,
            callBack = { callType, prames -> JSBridge(ynWebView,ynWebView.getWeb("")).callJS(null, null, 0, callType, prames) })
        onloadUrl()
        registerBc()
    }

    override fun onRestart() {
        Log.d("webViewActivity","Life with onRestart..")
        super.onRestart()
        JSBridge.sendJS(ynWebView,"PI_App",ON_APP_RESUMED, arrayOf("App进入前台"))
    }

    override fun onResume() {
        Log.d("webViewActivity","Life with onResume..")
        super.onResume()
        //下一次打开钱包
        val js = JSONObject()
        js.put("iswallet","true")
        FileUtil.writeFile(chooceAcPath, js.toString().toByteArray(), false)
    }

    override fun onBackPressed() {
        Log.d("webViewActivity","Life with onBackPressed..")
        JSBridge.sendJS(ynWebView,"PI_Activity",ON_BACK_PRESSED, arrayOf("App进入后台"))
    }

    override fun onDestroy() {
        Log.d("webViewActivity","Life with onDestroy..")
        unbindService(conn)
        unregisterReceiver(mReceiver)
        super.onDestroy()
    }

    //==========delegate========
    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        ynWebView.iterationOnActivityResult(requestCode, resultCode, data)
        super.onActivityResult(requestCode, resultCode, data)
    }


    //==========public============
    fun webViewBindService(message: String){
        if (ps != null){
            val ms = "window['handle_native_event']('ServiceAction', 'bind','$tag','$message')"
            ps!!.sendMessage(tag, ms)
        }
    }

    fun webViewPostMessage(sender: String, message: String){
        if (ps != null){
            val ms = "window.onWebViewPostMessage('$sender','$message')"
            ps!!.sendMessage(tag, ms)
        }
    }

    //==========private==========
    private fun rsgService(){
        val pkgName = applicationContext.packageName
        val intentName = "$pkgName.piservice"
        Log.d("framework", "WebViewActivity: Package Name = $pkgName, Intent Name = $intentName")
        val intent = Intent(intentName)
        intent.setPackage(pkgName)
        bindService(intent, conn, BIND_AUTO_CREATE)
    }


    private fun bindYnWebView(){
        ynWebView.createYnWebView(this)
        YNWebView.addWithName(tag!!, ynWebView)
        addJEV(this)
    }


    private fun registerBc() {
        val intentFilter = IntentFilter()
        intentFilter.addAction("send_message$tag")
        registerReceiver(mReceiver, intentFilter)
    }

    private fun onloadUrl(){
        //获取当前app版本号
        val pm = this.applicationContext.packageManager
        val info = pm.getPackageInfo(this.applicationContext.packageName, 0)
        val name = info.versionName
        //判断版本号
        val versionPath = mJsIntercept.apkPath + "/apkversion.txt"
        val version = FileUtil.readFile(versionPath)
        if (version == ""){
            //如果documents中没有版本号文件，将版本号文件写入documents中
            mJsIntercept.name = name.toString()
            FileUtil.writeFile(versionPath,name.toByteArray(),false)
        }else{
            //如果documents中有版本号文件，将版本号与当前app进行对比
            if (name != version){
                val htmlFile = File(mJsIntercept.htmlPath)
                FileUtil.RecursionDeleteFile(htmlFile)
                mJsIntercept.isUpdate = 1
                mJsIntercept.name = name.toString()
                FileUtil.writeFile(versionPath,name.toByteArray(),false)
            }
        }
        if(url == null || url == "") {
            url = resources.getString(URL_RES_ID)
        }
        isWallet = url == resources.getString(URL_RES_ID)
        Log.d("wabUrl","$url")
        if (url!!.startsWith("/")) {
            var content = FileUtil.readFile(mJsIntercept.htmlPath + url)
            if (content == "") {
                val stream = this.getAssets().open(url!!.substring(1))
                content = FileUtil.readFile(stream)
            }
            if (content != "") {
                super.loadDataWithBaseUrl("file:///android_asset" + url, content);
            } else {
                Log.d("JSIntercept", "loadUrl Error!!!");
            }
        } else {
            super.loadUrl(url!!)
        }
    }


    private val mReceiver = object : BroadcastReceiver() {
        override fun onReceive(context: Context?, intent: Intent?) {
            val action = intent?.action ?: return
            when (action) {
                "send_message$tag" -> {
                    val rpc = intent.getStringExtra("rpc")
                    val message = intent.getStringExtra("message")
                    val sender = intent.getStringExtra("from_web_view")
                    if (rpc == "true" ){
                        val callFun = String.format("javascript:window.onWebViewPostMessage('%s','%s')", sender, message)
                        ynWebView.evaluateJavascript(callFun)
                    }else{
                        ynWebView.evaluateJavascript(message)
                    }
                }
            }
        }
    }

    inner class webViewConn(): ServiceConnection{
        override fun onServiceConnected(name: ComponentName?, service: IBinder?) {
            ps = piservice.Stub.asInterface(service)
            ps!!.onMessage(tag, object : piserviceCallBack.Stub(){
                override fun sendMessage(statuCode: Int, message: String?) {
                    Log.d("webViewBindService", "statuCode$statuCode"+"message$message")
                    var ms = ""
                    if (statuCode == 200){
                        ms = "javascript:window.pi_sdk.piService.onBindService(undefined, '$message')"
                    }else if(statuCode == 400){
                        ms = "javascript:window.pi_sdk.piService.onBindService({code: -4, reason: $message},undefined)"
                    }else if (statuCode == 600){
                        var newMessage = TextUtil.resetSpecialCharInString(message!!)
                        Log.d("piservice","javascript:$newMessage")
                        ms = String.format("javascript:window.onWebViewPostMessage('%s','%s')", "JSVM", newMessage)
                    }
                    runOnUiThread { ynWebView.evaluateJavascript(ms) }
                }
            })
        }

        override fun onServiceDisconnected(name: ComponentName?) {

        }
    }

    companion object {
        const val APP_RESULT_CODE = 912
        val URL_RES_ID = R.string.init_url
        // val WebviewName = resources.getString(URL_RES_ID)
    }
}



