package com.high.pi_framework.module

import android.Manifest
import android.app.AlertDialog
import android.util.Log
import com.github.dfqin.grantor.PermissionListener
import com.github.dfqin.grantor.PermissionsUtil
import com.high.pi_framework.base.BaseJSModule
import com.high.pi_framework.webview.YNWebView
import com.quicksdk.*
import com.quicksdk.notifier.InitNotifier
import com.quicksdk.entity.GameRoleInfo
import com.quicksdk.entity.UserInfo
import com.quicksdk.notifier.LoginNotifier
import com.quicksdk.notifier.LogoutNotifier
import com.quicksdk.notifier.SwitchAccountNotifier
import com.quicksdk.notifier.PayNotifier
import com.quicksdk.notifier.ExitNotifier
import org.json.JSONObject
import com.quicksdk.entity.OrderInfo



class QuickSDKInterface(ynWebView: YNWebView) : BaseJSModule(ynWebView) {

    fun setInitNotifier(callBack:(callType: Int, prames: Array<Any>)->Unit){
        QuickSDK.getInstance().initNotifier = object : InitNotifier {
            override fun onSuccess() {
                //初始化成功
                callBack(CALLBACK, arrayOf(1, ""))
            }

            override fun onFailed(message: String, trace: String) {
                //初始化失败
                callBack(CALLBACK, arrayOf(0, message, trace))
            }
        }
    }

    fun setLoginNotifier(callBack: (callType: Int, prames: Array<Any>) -> Unit){
        QuickSDK.getInstance().loginNotifier = object : LoginNotifier {
            override fun onSuccess(userInfo: UserInfo) {
                //登录成功，获取到用户信息userInfo
                //通过userInfo中的UID、token做服务器登录认证
                val uid = userInfo.uid
                val userName = userInfo.userName
                val token = userInfo.token
                callBack(CALLBACK, arrayOf(1, uid, userName, token))
            }

            override fun onCancel() {
                //登录取消
                callBack(CALLBACK, arrayOf(0, "登录取消", ""))
            }

            override fun onFailed(message: String, trace: String) {
                //登录失败
                callBack(CALLBACK, arrayOf(-1, message, trace))
            }
        }
    }

    fun setLogoutNotifier(callBack: (callType: Int, prames: Array<Any>) -> Unit){
        QuickSDK.getInstance().logoutNotifier = object : LogoutNotifier {
            override fun onSuccess() {
                //注销成功
                callBack(CALLBACK, arrayOf(1, "注销成功"))
            }

            override fun onFailed(message: String, trace: String) {
                //注销失败，不做处理
                callBack(CALLBACK, arrayOf(0,"注销失败"))
            }
        }
    }

    fun setSwitchAccountNotifier(callBack: (callType: Int, prames: Array<Any>) -> Unit){
        QuickSDK.getInstance().switchAccountNotifier = object : SwitchAccountNotifier {
            override fun onSuccess(p0: UserInfo?) {
                val uid = p0!!.uid
                val userName = p0.userName
                val token = p0.token
                callBack(CALLBACK, arrayOf(1, uid, userName, token))
            }

            override fun onCancel() {
                callBack(CALLBACK, arrayOf(0, "取消切换账号","none"))
            }

            override fun onFailed(p0: String?, p1: String?) {
                callBack(CALLBACK, arrayOf(-1, p0!!, p1!!))
            }
        }
    }

    fun setPayNotifier(callBack: (callType: Int, prames: Array<Any>) -> Unit){
        QuickSDK.getInstance().payNotifier = object : PayNotifier {
            override fun onSuccess(sdkOrderID: String, cpOrderID: String, extrasParams: String) {
                //支付成功
                //sdkOrderID:quick订单号 cpOrderID：游戏订单号
                callBack(CALLBACK, arrayOf(1, sdkOrderID, cpOrderID, extrasParams))
            }

            override fun onCancel(cpOrderID: String) {
                //支付取消
                callBack(CALLBACK, arrayOf(0, cpOrderID))
            }

            override fun onFailed(cpOrderID: String, message: String, trace: String) {
                //支付失败
                callBack(CALLBACK, arrayOf(-1, cpOrderID, message, trace))
            }
        }
    }

    fun setExitNotifier(callBack: (callType: Int, prames: Array<Any>) -> Unit){
        QuickSDK.getInstance().exitNotifier = object : ExitNotifier {
            override fun onSuccess() {
                //退出成功，游戏在此做自身的退出逻辑处理
                callBack(CALLBACK, arrayOf(1, "退出成功"))
            }

            override fun onFailed(message: String, trace: String) {
                //退出失败，不做处理
                callBack(CALLBACK, arrayOf(0, message, trace))
            }
        }
    }

    fun initSDK(productCode: String, productKey: String, callBack: (callType: Int, prames: Array<Any>) -> Unit){
        PermissionsUtil.requestPermission(ctx!!, object : PermissionListener {
            override fun permissionGranted(permission: Array<String>) {
                ctx!!.runOnUiThread {  Sdk.getInstance().init(ctx, productCode, productKey) }
                callBack(SUCCESS, arrayOf(""))
            }

            override fun permissionDenied(permission: Array<String>) {
                callBack( FAIL, arrayOf("用户拒绝了权限!"))
            }
        }, arrayOf(Manifest.permission.WRITE_EXTERNAL_STORAGE, Manifest.permission.READ_PHONE_STATE), true, mTipInfo)
    }

    fun login(callBack: (callType: Int, prames: Array<Any>) -> Unit){
        ctx!!.runOnUiThread { User.getInstance().login(ctx) }
    }

    fun setGameRoleInfo(roleString: String, isCreate: Int, callBack: (callType: Int, prames: Array<Any>) -> Unit){
        val json = JSONObject(roleString)
        Log.d("z1u24","roleString = $roleString")
        val create = (isCreate == 1)
        val roleInfo = GameRoleInfo()
        roleInfo.serverID = json.getString("serverID")
        roleInfo.serverName = json.getString("serverName")
        roleInfo.gameRoleName = json.getString("gameRoleName")
        roleInfo.gameRoleID = json.getString("gameRoleID")
        roleInfo.gameBalance = json.getString("gameBalance")
        roleInfo.vipLevel = json.getString("vipLevel")
        roleInfo.setGameUserLevel(json.getString("gameUserLevel"))
        roleInfo.partyName = json.getString("partyName")
        roleInfo.roleCreateTime = json.getString("roleCreateTime")
        roleInfo.partyId = json.getString("partyId")
        roleInfo.gameRoleGender = json.getString("gameRoleGender")
        roleInfo.gameRolePower = json.getString("gameRolePower")
        roleInfo.partyRoleId = json.getString("partyRoleId")
        roleInfo.partyRoleName = json.getString("partyRoleName")
        roleInfo.professionId = json.getString("professionId")
        roleInfo.profession = json.getString("profession")
        roleInfo.friendlist = json.getString("friendlist")
        ctx!!.runOnUiThread { User.getInstance().setGameRoleInfo(ctx, roleInfo, create) }
    }

    fun payment(roleString: String, orderString: String, callBack: (callType: Int, prames: Array<Any>) -> Unit){
        val role = JSONObject(roleString)
        Log.d("z1u24","roleString = $roleString")
        val order = JSONObject(orderString)
        Log.d("z1u24","orderString = $orderString")
        val roleInfo = GameRoleInfo()
        roleInfo.serverID = role.getString("serverID")
        roleInfo.roleCreateTime =                role.getString("roleCreateTime")
        roleInfo.serverName = role.getString("serverName")
        roleInfo.gameRoleName = role.getString("gameRoleName")
        roleInfo.gameRoleID = role.getString("gameRoleID")
        roleInfo.setGameUserLevel(role.getString("gameUserLevel"))
        roleInfo.vipLevel = role.getString("vipLevel")
        roleInfo.gameBalance = role.getString("gameBalance")
        roleInfo.partyName = role.getString("partyName")
        val orderInfo = OrderInfo()
        orderInfo.cpOrderID = order.getString("cpOrderID")
        orderInfo.price = order.getDouble("price")
        orderInfo.goodsName = order.getString("goodsName")
        orderInfo.count = order.getInt("count")
        orderInfo.goodsDesc = order.getString("goodsDesc")
        orderInfo.amount = order.getDouble("amount")
        orderInfo.goodsID = order.getString("goodsID")
        orderInfo.extrasParams = order.getString("extrasParams")
        ctx!!.runOnUiThread { Payment.getInstance().pay(ctx, orderInfo, roleInfo) }
    }

    fun logout(callBack: (callType: Int, prames: Array<Any>) -> Unit){
        ctx!!.runOnUiThread {  User.getInstance().logout(ctx) }
    }

    fun exit(callBack: (callType: Int, prames: Array<Any>) -> Unit){
        ctx!!.runOnUiThread {
            if (QuickSDK.getInstance().isShowExitDialog) {
                Sdk.getInstance().exit(ctx)
            } else {
                // 游戏调用自身的退出对话框，点击确定后，调用quick的exit接口
                AlertDialog.Builder(ctx).setTitle("退出").setMessage("是否退出游戏?")
                    .setPositiveButton("确定") { _, _ -> Sdk.getInstance().exit(ctx) }
                    .setNegativeButton("取消", null).show()
            }
        }
    }

    fun getChannelType(callBack: (callType: Int, prames: Array<Any>) -> Unit){
        ctx!!.runOnUiThread {
            val channel = Extend.getInstance().channelType
            callBack(SUCCESS, arrayOf(channel))
        }
    }

}