package com.high.pi_framework.module

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.text.TextUtils
import android.util.Log
import com.high.pi_framework.Util.FileUtil
import com.high.pi_framework.Util.MemoryUtil
import com.high.pi_framework.Util.ViewUtil
import com.high.pi_framework.base.BaseJSModule
import com.high.pi_framework.framework.NewWebViewActivity
import com.high.pi_framework.framework.WebViewActivity
import com.high.pi_framework.webview.YNWebView
import com.high.pi_service.utils.TextUtil
import okhttp3.*
import org.json.JSONObject
import java.io.BufferedWriter
import java.io.File
import java.io.FileWriter
import java.io.IOException
import java.util.HashMap
import java.util.concurrent.TimeUnit

internal class FreeWebView(private val view: Any) : Runnable {
    override fun run() {
        YNWebView.destroyWebView(view)
    }
}

internal class NewWebView(private val context: Context, private val webViewName: String, private val url: String, private val headers: Map<*, *>, private  val injectContent: String, private val ynWebView: YNWebView, private val defaultName: String) : Runnable {

    override fun run() {
        WebViewManager.addNoShowView(this.webViewName, YNWebView.createWebView(context,this.webViewName,url,headers, injectContent, ynWebView, defaultName))
    }
}

/**
 * Created by iqosjay@gmail.com on 2018/11/7
 */
class WebViewManager constructor(ynWebView: YNWebView) : BaseJSModule(ynWebView) {

    /**
     * Get the webView's name by webView.
     *
     * @return The name of webView.
     */
    private val nameByWebViewObj: String?
        get() {
            val obj = yn.getWeb("")
            val entries = GAME_VIEW.entries
            for ((key, value) in entries) {
                if (obj == value) {
                    return key
                }
            }
            return null
        }


    /**
     * 新开webview，但不显示出来
     * @param callbackId
     * @param webViewName
     * @param url
     * @param headers，以 key1:value1 key2:value2 方式传递
     */
    fun newView(webViewName: String, url: String, headers: String, injectContent: String ,callBack:(callType: Int, prames: Array<Any>)->Unit) {

        if (TextUtils.isEmpty(webViewName)) {
            callBack(BaseJSModule.FAIL, arrayOf("The WebViews name canot be null."))
            return
        }

        if (TextUtils.isEmpty(url)) {
            callBack(BaseJSModule.FAIL, arrayOf("The url canot be null."))
            return
        }

        if (isNoShowViewExists(webViewName)) {
            callBack(BaseJSModule.FAIL, arrayOf("WebView name is exist."))
            return
        }

        val extraHeaders = mutableMapOf<Any, Any>()
        try {
            val obj = JSONObject(headers)
            val keysItr = obj.keys()
            while (keysItr.hasNext()) {
                val key = keysItr.next()
                extraHeaders.put(key, obj.getString(key))
            }
        } catch (e: Exception) {

        }
        val activity = yn.getEnv(yn.ACTIVITY) as Activity
        activity.runOnUiThread(NewWebView(activity.applicationContext, webViewName, url, extraHeaders, injectContent, yn, nameByWebViewObj!!))
        callBack(BaseJSModule.SUCCESS, arrayOf(""))
    }



    fun freeView(webViewName: String,callBack:(callType: Int, prames: Array<Any>)->Unit) {
        try {
            if ("default" == webViewName) {
                callBack(BaseJSModule.FAIL, arrayOf("The default WebView couldnot remove,please select a new one."))
                return
            }

            if (!isNoShowViewExists(webViewName)) {
                return
            }

            val view = WebViewManager.getNoShowView(webViewName)
            val activity = yn.getEnv(yn.ACTIVITY) as Activity
            activity.runOnUiThread(FreeWebView(view!!))
            WebViewManager.removeNoShowView(webViewName)
        } catch (e: Exception) {
            e.printStackTrace()
        }
        callBack(BaseJSModule.SUCCESS, arrayOf(""))
    }

    fun openWebView(webViewName: String, url: String, title: String, injectContent: String, screenOrientation: String, callBack:(callType: Int, prames: Array<Any>)->Unit) {
        if (TextUtils.isEmpty(webViewName)) {
            callBack(FAIL, arrayOf("The WebViews name can not be null."))
            return
        }
        if (TextUtils.isEmpty(url)) {
            callBack(FAIL, arrayOf("The url can not be null."))
            return
        }
        if (isGameViewExists(webViewName) && GAME_NAME == webViewName) {
            GAME_NAME = webViewName
            val intent = Intent(ctx, NewWebViewActivity::class.java)
            intent.addFlags(Intent.FLAG_ACTIVITY_REORDER_TO_FRONT)
            intent.putExtra("tag", webViewName)
            ctx!!.startActivity(intent)
            if (MemoryUtil.getTotalMemory() < 4.00) ctx!!.finish()
        } else {
            val intent = Intent(ctx, NewWebViewActivity::class.java)
            intent.addFlags(Intent.FLAG_ACTIVITY_REORDER_TO_FRONT)
            intent.putExtra("uagent", "YINENG_ANDROID_GAME/1.0")
            intent.putExtra("inject", "")
            intent.putExtra("screenOrientation", screenOrientation)
            intent.putExtra("title", title)
            intent.putExtra("load_url", url)
            intent.putExtra("tag", webViewName)
            downloadGameHtml(url){ code, content ->
                intent.putExtra("code", code)
                if (code == 0){
                    val file = "${ctx!!.filesDir}/game_value/index.html"
                    FileUtil.writeFile(file,content.toByteArray(),false)
                    intent.putExtra("filePath", file)
                }
                ctx!!.startActivity(intent)
            }
        }
    }

    /**
     * Open a new WebView to display a web page.
     *
     * @param callbackId  TS callbackId.
     * @param webViewName This is both name and key for the new webView,
     * you could called[.closeWebView] with this key
     * after opened the webView.
     * @param url         The web page's url.
     * @param title       The title what would you like to show in the new View.
     */
    fun openBookWebView(webViewName: String, url: String, injectContent: String, headers: String, callBack:(callType: Int, prames: Array<Any>)->Unit) {
        if (TextUtils.isEmpty(webViewName)) {
            callBack(BaseJSModule.FAIL, arrayOf("The WebViews name can not be null."))
            return
        }
        if (TextUtils.isEmpty(url)) {
            callBack(BaseJSModule.FAIL, arrayOf("The url can not be null."))
            return
        }
        else {
            if (!isNoShowViewExists(webViewName)) newView( webViewName, url, headers, injectContent, callBack )
            else callBack(BaseJSModule.FAIL, arrayOf("The WebViews name can not be null."))
        }
    }

    /**
     * Close the specified WebView.
     *
     * @param callbackId  TS callbackId.
     * @param webViewName WebView's name.
     */
    fun closeWebView(webViewName: String, callBack:(callType: Int, prames: Array<Any>)->Unit) {
        if ("default" == webViewName) {
            callBack(BaseJSModule.FAIL, arrayOf("The default WebView could not remove,please select a new one."))
            return
        }
        if (isGameViewExists(webViewName)) {
            GAME_NAME = ""
            sendCloseWebViewMessage(webViewName)
        }else if (isNoShowViewExists(webViewName)){
            freeView(webViewName, callBack)
        }else{
            return
        }
    }

    fun minWebView(webViewName: String, walletUrl: String, callBack:(callType: Int, prames: Array<Any>)->Unit) {
        if(ctx is NewWebViewActivity) (ctx as NewWebViewActivity).minsizeActivity(webViewName, walletUrl)
    }

    fun postWebViewMessage(webViewName: String, message: String, callBack:(callType: Int, prames: Array<Any>)->Unit) {
        val newMessage = TextUtil.resetSpecialCharInString(message)
        Log.d("postWebViewMessage","$message")
        if(webViewName == "JSVM"){
            Log.d("postWebViewMessage","now i get into JSVM Condition")
            val fromWebView = nameByWebViewObj
            if (ctx is WebViewActivity){
                (ctx as WebViewActivity).webViewPostMessage(fromWebView!!, newMessage)
            }else if (ctx is NewWebViewActivity){
                (ctx as NewWebViewActivity).webViewPostMessage(fromWebView!!, newMessage)
            }
        }else if(isGameViewExists(webViewName) || webViewName.equals("wallet")){
            Log.d("postWebViewMessage","now i get into else if Condition")
            val fromWebView = nameByWebViewObj
            val intent = Intent("send_message$webViewName")
            intent.putExtra("message", message)
            intent.putExtra("rpc", "true")
            intent.putExtra("from_web_view", fromWebView)
            ctx!!.sendBroadcast(intent)
        }
    }

    /**
     * Send a message to the web page view with the specified name.
     *
     * @param webViewName The name of WebView which you want send message to.
     * @param message     The message what you would like to send.
     */
    fun postReciptMessage(webViewName: String, message: String, isRPC: String,callBack:(callType: Int, prames: Array<Any>)->Unit) {
        val newMessage = TextUtil.resetSpecialCharInString(message)
        if (!isGameViewExists(webViewName) && !isNoShowViewExists(webViewName)) {
            callBack(BaseJSModule.FAIL, arrayOf("The WebView's name is not exists."))
            return
        }else if(isGameViewExists(webViewName)){
            val fromWebView = nameByWebViewObj
            val intent = Intent("send_message$webViewName")
            intent.putExtra("message", newMessage)
            intent.putExtra("rpc", isRPC)
            intent.putExtra("from_web_view", fromWebView)
            ctx!!.sendBroadcast(intent)
        }else if (isNoShowViewExists(webViewName)){
            YNWebView.evaluateJavascript(ctx!!,getNoShowView(webViewName) as Any,newMessage,isRPC,nameByWebViewObj!!)
        }
    }

    fun reload(callBack:(callType: Int, prames: Array<Any>)->Unit) {
        val activity = yn.getEnv(yn.ACTIVITY)
        if (activity is NewWebViewActivity) {
            downloadGameHtml(activity.url!!){ code, content ->
                if(code == 0){
                    activity.injectContent = content
                    activity.loadUrlImpl(true)
                }else{
                    activity.loadUrlImpl(false)
                }
            }
        }
    }

    fun getScreenModify(callBack:(callType: Int, prames: Array<Any>)->Unit) {
        val activity = yn.getEnv(yn.ACTIVITY) as Activity
        val high = ViewUtil.getStatusBarHeight(activity)
        callBack(BaseJSModule.SUCCESS, arrayOf(high,0))
    }

    /**
     * Close the WebView by webView's name.Finished by send broadcast,
     * because in this way, the coupling is the lowest.
     *
     * @param webViewName WebView's name.
     */
    private fun sendCloseWebViewMessage(webViewName: String) {
        val intent = Intent("close_web_view")
        intent.putExtra("web_view_name", webViewName)
        ctx!!.sendBroadcast(intent)
    }

    private fun downloadGameHtml(url:String, callBack:(code: Int, content: String) -> Unit){
        val timeout = ctx!!.resources.getString(com.high.pi_framework.R.string.down_timeout).toInt()
        val okHttpClient = OkHttpClient.Builder().connectTimeout(timeout.toLong(), TimeUnit.SECONDS).writeTimeout(timeout.toLong(), TimeUnit.SECONDS).readTimeout(timeout.toLong(),
            TimeUnit.SECONDS).build()
        val request = Request.Builder().url(url).get().build()
        val call = okHttpClient.newCall(request)
        call.enqueue(object : Callback {
            override fun onResponse(call: Call?, response: Response?) {
                if(response!!.code() == 200){
                    val context = response.body()!!.string()
                    callBack(0,context)
                }else{
                    callBack(-1,"")
                }
            }
            override fun onFailure(call: Call?, e: IOException?) {
                callBack(-1,"")
            }
        })
    }



    companion object {
        /**
         * All WebViews that have been opened.
         */
        private val NOSHOW_VIEW = HashMap<String, Any>()

        private var GAME_NAME = ""

        private var GAME_VIEW = HashMap<String, Any>()

        /**
         * Check whether the name of webView is exists.
         *
         * @param webViewName The name of webView
         * @return true for the name has been exists.
         */
        fun isGameViewExists(webViewName: String): Boolean {
            for (key in GAME_VIEW.keys) {
                if (webViewName == key) {
                    return true
                }
            }
            return false
        }

        /**
         * Add WebView.
         *
         * @param key    WebView's name.
         * @param object WebView:X5 or Android
         */
        fun addGameView(key: String, `object`: Any) {
            GAME_VIEW[key] = `object`
        }

        /**
         * Remove WebView by its name.
         *
         * @param key WebView's name
         */
        fun removeGameView(key: String) {
            GAME_VIEW.remove(key)
        }

        fun getGameView(key: String): Any? {
            return GAME_VIEW[key]
        }


        fun isNoShowViewExists(webViewName: String): Boolean {
            for (key in NOSHOW_VIEW.keys) {
                if (webViewName == key) {
                    return true
                }
            }
            return false
        }

        /**
         * Add WebView.
         *
         * @param key    WebView's name.
         * @param object WebView:X5 or Android
         */
        fun addNoShowView(key: String, `object`: Any) {
            NOSHOW_VIEW[key] = `object`
        }

        /**
         * Remove WebView by its name.
         *
         * @param key WebView's name
         */
        fun removeNoShowView(key: String) {
            NOSHOW_VIEW.remove(key)
        }

        fun getNoShowView(key: String): Any? {
            return NOSHOW_VIEW[key]
        }

    }
}