package com.high.pi_service;

import android.os.Handler;
import android.os.Looper;
import okhttp3.Response;
import okhttp3.WebSocket;
import okhttp3.WebSocketListener;
import okhttp3.mockwebserver.MockResponse;
import okhttp3.mockwebserver.MockWebServer;
import okio.ByteString;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import com.high.pi_service.piLog.Log;
import com.yineng.piv8.V8;
import com.yineng.piv8.utils.V8Inspector;

import java.io.IOException;
import java.net.InetAddress;

public class V8WebServer extends V8Inspector {

    private static String TAG = "piv8";
    private V8 runtime;
    private int port = 41868;
    private Handler mainHandler;
    private static V8WebServer v8WebServer = null;
    private String ipv4Address = null;

    public static V8WebServer getInstance(V8 v8, String ipv4Address){
        if (v8WebServer == null){
            v8WebServer = new V8WebServer(v8,ipv4Address);
        }
        return v8WebServer;
    }

    public static V8WebServer getV8WebServer(){
        return v8WebServer;
    }

    public static void close(){
        if(v8WebServer != null){
            v8WebServer = null;
        }
    }

    public V8WebServer(V8 v8, String ipAddress){
        this.runtime = v8;
        this.ipv4Address = ipAddress;
        mainHandler = new Handler(Looper.getMainLooper());
        createWebSocketServer();
    }


    private void createWebSocketServer(){
        new Thread(new Runnable() {
            @Override
            public void run() {
                MockWebServer mockWebServer = new MockWebServer();
                mockWebServer.enqueue(new MockResponse().withWebSocketUpgrade(new WebSocketListener() {
                    @Override
                    public void onOpen(final WebSocket webSocket, Response response) {
                        super.onOpen(webSocket, response);
                        Log.INSTANCE.d(Log.INSTANCE.getSERVICE(), "V8WebServer","createWebSocketServer", "mock webServer is onopen");
                        mainHandler.post(new Runnable() {
                            @Override
                            public void run() {
                                runtime.disconnect();
                                runtime.connect(webSocket);
                            }
                        });
                    }

                    @Override
                    public void onMessage(WebSocket webSocket, final String text) {
                        super.onMessage(webSocket, text);
                        Log.INSTANCE.d(Log.INSTANCE.getSERVICE(), "V8WebServer","createWebSocketServer", "mock webServer get a message: text = " + text);
                        mainHandler.post(new Runnable() {
                            @Override
                            public void run() {
                                runtime.dispatchMessage(text);
                            }
                        });
                    }

                    @Override
                    public void onMessage(WebSocket webSocket, ByteString bytes) {
                        super.onMessage(webSocket, bytes);
                        Log.INSTANCE.d(Log.INSTANCE.getSERVICE(), "V8WebServer","createWebSocketServer", "mock webServer get a message: bytes");
                    }

                    @Override
                    public void onClosed(WebSocket webSocket, int code, String reason) {
                        super.onClosed(webSocket, code, reason);
                        Log.INSTANCE.d(Log.INSTANCE.getSERVICE(), "V8WebServer","createWebSocketServer", "mock webServer is onclosed");
                        mainHandler.post(new Runnable() {
                            @Override
                            public void run() {
                                runtime.disconnect();
                            }
                        });
                    }

                    @Override
                    public void onFailure(WebSocket webSocket, Throwable t, Response response) {
                        super.onFailure(webSocket, t, response);
                        Log.INSTANCE.d(Log.INSTANCE.getSERVICE(), "V8WebServer","createWebSocketServer", "mock webServer is onFailure");
                    }
                }));
                try {
                    mockWebServer.start(InetAddress.getByName(ipv4Address),port);
                    Log.INSTANCE.d(Log.INSTANCE.getSERVICE(), "V8WebServer","createWebSocketServer", "start debugger port:" + ipv4Address + "=====post: " + port);
                } catch (IOException e) {
                    Log.INSTANCE.d(Log.INSTANCE.getSERVICE(), "V8WebServer","createWebSocketServer", "start debugger port:" + ipv4Address + "=====post: " + port +" failed with "+ e.getMessage());
                }
            }
        }).start();
    }

    @Override
    public void send(Object connection, String message) {
        super.send(connection, message);
        WebSocket ws = (WebSocket)connection;
        Log.INSTANCE.d(Log.INSTANCE.getSERVICE(), "V8WebServer","send", "send message to chrome: " + message);
        ws.send(message);
    }

    @Override
    public void sendToDevToolsConsole(Object connection, String message, String level) {
        super.sendToDevToolsConsole(connection, message, level);
        try{
            JSONObject consoleMessage = new JSONObject();
            JSONObject params = new JSONObject();
            params.put("type", level);
            params.put("executionContextId", 0);
            params.put("timestamp", 0.000000000000000);
            JSONArray args = new JSONArray();
            args.put(message);
            params.put("args", args);
            consoleMessage.put("method", "Runtime.consoleAPICalled");
            consoleMessage.put("params", params);
            String sendingText = consoleMessage.toString();
            send(connection,sendingText);
        }catch (JSONException e){
            e.printStackTrace();
        }
    }

}
