package com.high.pi_service

import android.content.Context
import android.os.Handler
import android.os.Looper
import com.high.pi_service.piLog.Log
import com.high.pi_service.utils.DataHandleManager
import com.high.pi_service.utils.FileUtil
import com.yineng.piv8.V8
import com.yineng.piv8.V8Array
import com.yineng.piv8.V8Function
import com.yineng.piv8.V8Object
import okhttp3.*
import org.json.JSONArray
import org.json.JSONObject
import java.io.BufferedReader
import java.io.IOException
import java.io.InputStreamReader
import java.util.concurrent.TimeUnit

class PiV8JsBootManager(private val context: Context, private val v8: V8){

    private val mainHander = Handler(Looper.getMainLooper())
    private val bootPath: String = "/data/data/" + context.packageName
    val htmlPath: String = "$bootPath/html/vm"
    val apkPath: String  = "$bootPath/apk_back"
    var update: Int = 0
    private var mBootFilePaths: JSONObject? = null
    private val mConfigPath: String = "$htmlPath/bootFilePaths.json"

    init {
        val context = FileUtil.readFile(mConfigPath)
        try {
            if(context == ""){
                mBootFilePaths = JSONObject()
            }else{
                mBootFilePaths = JSONObject(context)
            }
        }catch (e: Exception){
            e.printStackTrace()
        }
    }

    fun saveFile(path: String, content: String){
        var sPath = path
        try {
            if (sPath.contains(".depend")) {
                sPath = sPath.replace(".depend", "depend")
            }

            val fullPath = "$htmlPath/$sPath"
            FileUtil.writeFile(fullPath, content.toByteArray(Charsets.UTF_8), false)

            sPath = sPath.substring(sPath.lastIndexOf("/") + 1)
            mBootFilePaths!!.put(sPath, fullPath)
            FileUtil.writeFile(mConfigPath, mBootFilePaths!!.toString().toByteArray(Charsets.UTF_8),false)
            Log.d(Log.SERVICE,"piv8JsBootManager","saveFile","saveFile with path: $fullPath")
        } catch (e: Exception) {
            Log.e(Log.SERVICE,"piv8JsBootManager","saveFile","error: $e.message")
            e.printStackTrace()
        }
    }

    fun getMobileBootFiles(callback: V8Function){
        val cb = callback.twin()
        val result = JSONObject()
        try {
            val iterator = mBootFilePaths!!.keys()
            while (iterator.hasNext()) {
                val key = iterator.next() as String
                val fullPath = mBootFilePaths!!.getString(key)
                val content = FileUtil.readFile(fullPath)
                val dhi = DataHandleManager.createNewDataHandle()
                DataHandleManager.setContent(dhi,content,key)
                result.put(key, dhi)
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
        mainHander.post { val v8Array = V8Array(v8); v8Array.push(result.toString()); cb.call(null,v8Array); v8Array.close(); cb.close() }
    }

    fun restartJSVM(){
        if (context is piv8Service){
            context.vmRestart()
        }
        mainHander.post { JSVMManager.get().restartJSVM() }
    }

    fun loadJS(defaultDomains: String, path: String):Int{
        Log.d(Log.SERVICE,"piv8JsBootManager","loadJS","defaultDomains: $defaultDomains, path: $path")
        if(defaultDomains.contains("file:///android_asset")){
            var subPath = path.subSequence(1,path.length)
            try {
                val temp = context.getAssets().open(subPath.toString())
                val br = BufferedReader(InputStreamReader(temp))
                val sb = StringBuffer()
                var strLine = br.readLine()
                while (strLine  != null) {
                    sb.append(strLine).append("\r\n")
                    strLine = br.readLine()
                }
                br.close()
                val dh = DataHandleManager.createNewDataHandle()
                DataHandleManager.setContent(dh, sb.toString(), path)
                return dh
            }catch (e: Exception){
                return -1
            }
        }else{
            val fullPath = defaultDomains + path
            val content = FileUtil.readFile(fullPath)
            val dh = DataHandleManager.createNewDataHandle()
            DataHandleManager.setContent(dh,content,path)
            return dh
        }
    }

    fun updateApp(url: String){
        //下载app（JSVM不需要下载）
        Log.e(Log.SERVICE,"piv8JsBootManager","updateApp","piservice now not supported to update app")
    }

    fun getAppVersion(cb: V8Function){
        var name = ""
        try {
            val pm = context.applicationContext.packageManager
            val info = pm.getPackageInfo(context.applicationContext.packageName, 0)
            name = info.versionName
        } catch (e: Exception) {
            e.printStackTrace()
        }
        val arr = V8Array(v8)
        arr.push(true)
        arr.push(name)
        arr.push(update)
        cb.call(null,arr)
        arr.close()
    }

    fun updateFinish(){
        update = 0
    }

    fun updateDownload(domains: V8Array, url: String, files: String, success: V8Function, fail: V8Function, progress: V8Function){
        val domainList: MutableList<String> = mutableListOf()
        for (i in 0 until domains.length()){
            domainList.add(domains[i] as String)
        }
        val successCB = success.twin()
        val failCB = fail.twin()
        var start = 0
        var i = 0
        var obj = JSONArray(files)
        goDownload(domainList,url,obj, { file, dh ->
            start = start + 1
            mainHander.post {
                val arr = V8Array(v8)
                arr.push(file)
                arr.push(dh)
                successCB.call(null,arr)
                if (start >= files.length){
                    successCB.close()
                }
                arr.close()
                failCB.close()
            }
        }, {
            mainHander.post {
                val arr = V8Array(v8)
                arr.push("downLoad fail")
                failCB.call(null,arr)
                arr.close()
                failCB.close()
                successCB.close()
            }
        },{

        },0)
    }

    fun saveDepend(content: String){
        saveFile(".depend",content)
    }

    fun saveIndexJS(content: String){
        saveFile("boot/jsindex.js",content)
    }

    fun goDownload(domains: List<String>, url: String, files: JSONArray, successCallBack:(name: String, dh: Int)->Unit, failCallBack:()->Unit, progressCallBack:()->Unit, retry:Int){
        if (retry >= domains.count()){
            failCallBack()
            return
        }
        val http_url = domains[retry]
        val fullPath = http_url.toString() + url
        Log.d(Log.SERVICE,"PiV8JsBootManager","goDownload","fullpath: $fullPath")
        val okHttpClient = OkHttpClient.Builder().readTimeout(5,TimeUnit.SECONDS).build()
        val request = Request.Builder().url(fullPath).get().build()
        val call = okHttpClient.newCall(request)
        call.enqueue(object : Callback{
            override fun onFailure(call: Call, e: IOException) {
                goDownload(domains,url,files,successCallBack,failCallBack,progressCallBack,retry+1)
            }
            override fun onResponse(call: Call, response: Response) {
                if( response.code() != 200 ){
                    goDownload(domains,url,files,successCallBack,failCallBack,progressCallBack,retry+1)
                }else{
                    val isn = response.body()!!.bytes()
                    var start = 0
                    var end = 0
                    var i = 0
                    while ( i < files.length()){
                        var file = files[i] as JSONObject
                        start = end
                        end = end + file.get("size") as Int
                        var name = file.get("path").toString()
                        var data:ByteArray = isn.copyOfRange(start, end)
                        var content: String = data.toString(Charsets.UTF_8)
                        var dhi = DataHandleManager.createNewDataHandle()
                        DataHandleManager.setContent(dhi, content, name)
                        successCallBack(file.toString(), dhi)
                        if (name.contains("boot")){
                            //保存到documents
                            saveFile(name,content)
                        }
                        i = i + 1
                    }
                }
            }
        })
    }
}