package com.high.pi_service

import android.app.Service
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.os.*
import com.high.pi_service.piLog.Log
import com.high.pi_service.utils.FileUtil
import com.yineng.piv8.V8
import com.yineng.piv8.V8Array
import com.yineng.piv8.V8Function
import okhttp3.*
import org.json.JSONObject
import java.io.IOException
import java.util.concurrent.TimeUnit

class piv8Service: Service() {

    private val callBackManager: MutableMap<String, piserviceCallBack> = mutableMapOf()
    private val actionCallBack: MutableMap<String, V8Function> = mutableMapOf()
    private var isVMReady: Boolean = false
    private var runtime: V8? = null
        get() = JSVMManager.get().getRuntime()

    //================life=========
    override fun onCreate() {
        super.onCreate()
        registerBc()
        JSVMManager.get().createV8(this)
    }

    override fun onBind(intent: Intent?): IBinder? {
        val bind = piv8Bind()
        return bind
    }

    override fun onDestroy() {
        for (key in actionCallBack.keys){
            val cb = actionCallBack[key]
            if (!cb!!.isReleased){
                actionCallBack.remove(key)
                cb.close()
            }
        }
        JSVMManager.get().vmBridge!!.onDestroy()
        runtime?.close()
        unregisterReceiver(mBroadcastReceiver)
        super.onDestroy()
    }


    //===================private==============
    private fun registerBc() {
        val intentFilter = IntentFilter()
        intentFilter.addAction("outpay_action")
        intentFilter.addAction("inpay_action")
        intentFilter.addAction("share_action")
        intentFilter.addAction("ali_pay_action")
        intentFilter.addAction("wx_pay_action")
        intentFilter.addAction("close_vm")
        registerReceiver(mBroadcastReceiver, intentFilter)
    }

    fun vmReady(){
        Log.d(Log.SERVICE,"piv8Service","vmReady","vm service is load ok")
        isVMReady = true
    }

    fun vmRestart(){
        isVMReady = false
    }

    fun sendMessage(statuCode: Int, webViewName: String, message: String){
        Log.d(Log.SERVICE,"piv8Service","sendMessage","send message to webView statuCode: $statuCode, webViewName: $webViewName message: $message")
        if (webViewName == "all"){
            for (key in callBackManager.keys){
                val cb = callBackManager[key]
                cb!!.sendMessage(statuCode, message)
            }
        }else{
            if (webViewName in callBackManager.keys){
                val cb = callBackManager[webViewName]
                cb!!.sendMessage(statuCode, message)
            }
        }
    }



    private fun postMessage(webViewName: String?, message: String?){
        Log.d(Log.SERVICE,"piv8Service","postMessage","get message frome webView $webViewName,  message: $message, vm ready? $isVMReady")
        if (message != null && isVMReady == true){
            try {
                Handler(Looper.getMainLooper()).post {
                    runtime?.executeVoidScript(message)
                }
            }catch (e: Exception){
                if (webViewName in callBackManager.keys){
                    callBackManager[webViewName]!!.sendMessage(400, "there is a error hapened")
                }
            }
        }

    }



    //游戏内界面
    fun downLoadShareGameImageWithUrl(gameName: String, url: String, callBack: V8Function){
        Log.d(Log.SERVICE,"piv8Server","downLoadShareGameImageWithUrl","gameName is $gameName, url is $url")
        val cb = callBack.twin()
        var imageNameJson = JSONObject()
        val imageDocumentsPath = filesDir.absolutePath + "/imageDocuments/"
        val imageNameListPath = imageDocumentsPath + "iamgeNameList.json"
        val ms = String.format("%s%s.jpg",imageDocumentsPath,gameName)
        val content = FileUtil.readFile(imageNameListPath)
        if (content != ""){
            imageNameJson = JSONObject(content)
        }
        val iterator = imageNameJson!!.keys()
        while (iterator.hasNext()) {
            val key = iterator.next() as String
            val value = imageNameJson!!.getString(key)
            if (key.equals(gameName)) {
                if (url.equals(value)){
                    val arr = V8Array(runtime)
                    arr.push(0)
                    arr.push(ms)
                    callBack.call(null, arr)
                    arr.close()
                    cb.close()
                    return
                }else{
                    imageNameJson.remove(key)
                }
            }
        }
        val okHttpClient = OkHttpClient.Builder().readTimeout(5, TimeUnit.SECONDS).build()
        val request = Request.Builder().url(url).get().build()
        val call = okHttpClient.newCall(request)
        call.enqueue(object : Callback {
            override fun onFailure(call: Call, e: IOException) {
                Handler(Looper.getMainLooper()).post {
                    val arr = V8Array(runtime)
                    arr.push(-1)
                    arr.push("the file is download error")
                    cb.call(null, arr)
                    arr.close()
                    cb.close()
                }
            }
            override fun onResponse(call: Call, response: Response) {
                val code = response.code()
                if (code != 200){
                    Handler(Looper.getMainLooper()).post {
                        val arr = V8Array(runtime)
                        arr.push(-1)
                        arr.push("the file is download error")
                        cb.call(null, arr)
                        arr.close()
                        cb.close()
                    }
                }else{
                    FileUtil.writeFile(ms,response.body()!!.bytes(),false)
                    imageNameJson.put(gameName, url)
                    FileUtil.writeFile(imageNameListPath,imageNameJson!!.toString().toByteArray(Charsets.UTF_8),false)
                    Handler(Looper.getMainLooper()).post{
                        val arr = V8Array(runtime)
                        arr.push(0)
                        arr.push(ms)
                        cb.call(null, arr)
                        arr.close()
                        cb.close()
                    }
                }
            }
        })
    }


    fun goChareActivity(balance: Int){
        val activityName = applicationContext.packageName + ".gameView.ChargeActivity"
        val intent = Intent(activityName)
        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
        intent.putExtra("balance",balance)
        startActivity(intent)
    }

    fun goChareInGameActivity(orderId: String, kupayId: String, balance: Int, seller: String, price: String, pay: Int){
        val activityName = applicationContext.packageName + ".gameView.ChargeInGameActivity"
        val intent = Intent(activityName)
        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
        intent.putExtra("orderId",orderId)
        intent.putExtra("kupayId",kupayId)
        intent.putExtra("balance",balance)
        intent.putExtra("seller",seller)
        intent.putExtra("price",price)
        intent.putExtra("pay",pay)
        startActivity(intent)
    }

    fun goShare(imageName: String, imagePath: String, userName: String, shareCode: String, shareUrl: String, callBack: V8Function){
        val cb = callBack.twin()
        actionCallBack.put("share_action", cb)

        val pkgName = applicationContext.packageName
        val intentName = pkgName + ".gameView.ShareActivity"
        Log.d(Log.SERVICE,"piv8Server","goShare","piV8Server: Package Name = $pkgName, Intent Name = $intentName")
        val intent = Intent(intentName)
        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
        intent.putExtra("nickname",userName)
        intent.putExtra("codeURL",shareUrl)
        intent.putExtra("imagePath",imagePath)
        intent.putExtra("code",shareCode)
        intent.putExtra("imageName",imageName)
        startActivity(intent)
    }

    fun addActionListener(type: String, callBack: V8Function){
        val cb = callBack.twin()
        if (type in actionCallBack.keys){
            actionCallBack[type] = cb
        }else{
            actionCallBack.put(type, cb)
        }
    }

    fun removeActionListener(type: String){
        if (type in actionCallBack.keys){
            val cb = actionCallBack[type]
            if (!cb!!.isReleased){
                cb.close()
            }
            actionCallBack.remove(type)
        }
    }

    fun goWXPay(app_id: String, partnerid: String, prepayid: String, packages: String, noncestr: String, timestamp: String, sign: String, callBack: V8Function){

    }

    fun goAliPay(payInfo: String, callBack: V8Function){

    }

    private val mBroadcastReceiver = object : BroadcastReceiver(){
        override fun onReceive(context: Context?, intent: Intent?) {
            val action = intent?.action ?: return
            when(action){
                "outpay_action" -> {
                    //获取code
                    val code = intent.getIntExtra("code",0)
                    val way = intent.getStringExtra("pay_way")
                    val payAmount = intent.getIntExtra("payAmount",0)
                    if (action in actionCallBack.keys){
                        val cb = actionCallBack[action]
                        if (!cb!!.isReleased){
                            Log.d(Log.SERVICE,"piv8Server","outpay_action","get outpay_action with code: $code, way: $way, payAmount: $payAmount")
                            val arr = V8Array(runtime)
                            arr.push(code)
                            arr.push(payAmount)
                            arr.push(way)
                            cb.call(null, arr)
                            arr.close()
                        }
                    }
                }
                "inpay_action" -> {
                    val code = intent.getIntExtra("code",0)
                    val way = intent.getStringExtra("pay_way")
                    val orderId = intent.getStringExtra("orderId")
                    if (action in actionCallBack.keys){
                        val cb = actionCallBack[action]
                        if (!cb!!.isReleased){
                            val arr = V8Array(runtime)
                            arr.push(code)
                            arr.push(orderId)
                            arr.push(way)
                            cb.call(null, arr)
                            arr.close()
                        }
                    }
                }
                "share_action" -> {
                    val result = intent.getIntExtra("ruselt", 0)
                    if (action in actionCallBack.keys){
                        val cb = actionCallBack[action]
                        if (!cb!!.isReleased){
                            val arr = V8Array(runtime)
                            arr.push(result)
                            cb.call(null, arr)
                            arr.close()
                        }
                    }
                }
                "wx_pay_action" -> {
                    val result = intent.getIntExtra("ruselt", 0)
                    if (action in actionCallBack.keys){
                        val cb = actionCallBack[action]
                        if (!cb!!.isReleased){
                            val arr = V8Array(runtime)
                            arr.push(result)
                            cb.call(null, arr)
                            actionCallBack.remove(action)
                            cb.close()
                            arr.close()
                        }
                    }
                }
                "ali_pay_action" -> {
                    val result = intent.getIntExtra("ruselt", 0)
                    if (action in actionCallBack.keys){
                        val cb = actionCallBack[action]
                        if (!cb!!.isReleased){
                            val arr = V8Array(runtime)
                            arr.push(result)
                            cb.call(null, arr)
                            actionCallBack.remove(action)
                            cb.close()
                            arr.close()
                        }
                    }
                }
                "close_vm" -> {
                    Process.killProcess(Process.myPid())
                }
            }
        }
    }

    inner class piv8Bind(): piservice.Stub(){
        override fun onMessage(webViewName: String?, callBack: piserviceCallBack?){
            if (webViewName != null && callBack != null){
                callBackManager.put(webViewName, callBack)
            }
        }

        override fun sendMessage(webViewName: String?, message: String?){
            if (webViewName != null && message != null){
                postMessage(webViewName, message)
            }
        }

        override fun unbindWebView(webViewName: String?) {
            if (webViewName in callBackManager.keys){
                callBackManager.remove(webViewName)
            }
        }
    }

}








